
The goal of this version 5.0.0 is to clean up the API and make it consistent and more intuitive. To this end, we have a lot of deprecated features.

The overall API image will look like this:
* Inject your state using: `RM.inject`, `RM.injectFuture`, `RM.injectStream`, or `RM.injectFoo` (Foo replaces `Auth`, `Crud`, `Animation`, ...).
* Listen to your state to rebuild the UI using: `ReactiveStatelessWidget`, `OnReactive`, `OnBuilder`, or `OnFooBuilder` (Foo replaces `Auth`, `Crud`, `Animation`, ...).        
* Notify your state using: direct immutable mutation (myState.state = newState), `setState`, or `toggle` methods.
* Whenever you want to invoke side effects, on (state /or widget) initialization, mutation, or disposition, use `sideEffects` parameter.

## state injection:
### Before:

The API of side effects is not that clear.

```dart
final myState = RM.inject<MyState>(
  () => MyState(),
  onData: (data) {
    print(data);
  },
  onError: (err, refresh) {
    print(err);
  },
  onWaiting: () {
    print('onWaiting');
  },
  onDisposed: (_) {
    print('disposed');
  },
  onSetState: On(
    () {
      print('onSetState');
    },
  ),
);
```
### After
```dart
final myState = RM.inject<MyState>(
  () => MyState(),
  sideEffects: SideEffects(
    onSetState: (snap) {
      print('onSetState');
      if (snap.isWaiting) {
        print('onWaiting');
      } else if (snap.hasError) {
        print('error');
      } else if (snap.hasData) {
        print(snap.data);
      }

      //or simply use onAll or onOrElse
      // snap.onAll(
      //   onWaiting: () => print('onWaiting'),
      //   onError: (err, refresh) => print('Error'),
      //   onData: (data) => print(data),
      // );
    },
    dispose: () {
      print('disposed');
    },
  ),
);
```

`SideEffects` has named constructor to reduce the boilerplate in some use cases: `SideEffects.onData`, `SideEffects.onWaiting`, `SideEffects.onError`, `SideEffects.onAll`, `SideEffects.onOrElse`.

The same `SideEffects` class will be used in `OnReactive`, `onBuilder` and `setState` method.

`onDate`, `onError`, `onWaiting`, `onSetState` are `deprecated` in all injected methods. Use `SideEffects` instead.

## Builders (widget listeners)
To listen to an injected state you can:
* use `ReactiveStatelessWidget` or `OnReactive` for implicit subscription.
* use `OnBuilder` for explicit subscription (or use `rebuild` method equivalent of `OnBuilder` for explicit subscription).
* use `OnFooBuilder` for `injectedFoo` (Foo replaces `Auth`, `Crud`, `Animation`, ...).

All other `On` widget subscription is deprecated.

### Before
```dart
On(
  () => MyWidget(),
).listenTo(
  myState,
),
//
On.data(
  () => MyWidget(),
).listenTo(
  myState,
),
//
On.animation(
  (animate) => MyWidget(),
).listenTo(
  myAnimation,
),
```
### after
```dart
OnBuilder(
  listenTo: myState,
  builder: ()=> MyWidget(),
)
//
OnBuilder.data(
  listenTo: myState,
  builder: (data)=> MyWidget(),
)
//
OnAnimationBuilder(
  listenTo: myAnimation,
  builder: (animate) => MyWidget(),
)
```

## setState method.

### before
```dart
myState.setState(
  (s) => newState,
  onData: (data) {
    print(data);
  },
  onError: (error) {
    print(error);
  },
  onSetState: On(
    () {
      print('onSetState');
    },
  ),
  onRebuildState: () {
    print('onRebuildState');
  },
);
```

Side effects defined here can override side effects defined globally in `RM.inject` and other equivalent methods. The logic of overridden is implicitly built in states_rebuilder.

### after
```dart
myState.setState(
  (s) => newState,
  sideEffects: SideEffects(
    onSetState: (snap) {
      print('onSetState');
      if (snap.hasError) {
        print('error');
      } else if (snap.hasData) {
        print(snap.data);
      }
    },
    onAfterBuild: () {
      print('onRebuildState');
    },
  ),
  //Optionally
  shouldOverrideDefaultSideEffects: (snap)=> true,
);
``` 

In this version, you have to manual decide when to override the global default side effects using the `shouldOverrideDefaultSideEffects` parameter.